@@ -13,64 +13,52 @@ class ThreadedExecutor(
 ):
     def __init__(
         self,
-        worker: worker.Worker,
+        worker_object: worker.Worker,
         number_of_threads: int,
     ) -> None:
-        self.worker = worker
+        self.worker_object = worker_object
         self.number_of_threads = number_of_threads
 
-        has_soft_timeout = self.worker.config.timeouts.soft_timeout > 0
+        has_soft_timeout = self.worker_object.config.timeouts.soft_timeout > 0
         self.should_use_a_killer = has_soft_timeout
-        self.thread_killers: typing.Dict[int, killer.thread.Killer] = {}
+        self.thread_killer = killer.thread.Killer(
+            exception=worker.WorkerSoftTimedout,
+            sleep_interval=0.1,
+        )
+        self.interrupt_exception: typing.Optional[Exception] = None
 
     def execute_tasks(
         self,
         tasks: typing.Iterable[objects.Task],
     ) -> None:
-        interrupt_exception = None
-        future_to_task = {}
+        if self.should_use_a_killer:
+            self.thread_killer.start()
 
-        executor = concurrent.futures.ThreadPoolExecutor(
+        with concurrent.futures.ThreadPoolExecutor(
             max_workers=self.number_of_threads,
-        )
+        ) as executor:
+            running_futures = []
+            for task in tasks:
+                future = executor.submit(self.execute_task, task)
+                running_futures.append(future)
 
-        for task in tasks:
-            future = executor.submit(self.execute_task, task)
-            future_to_task[future] = task
+                if len(running_futures) == self.number_of_threads:
+                    finished, pending = concurrent.futures.wait(
+                        fs=running_futures,
+                        timeout=None,
+                        return_when=concurrent.futures.FIRST_COMPLETED,
+                    )
+                    for finished_future in finished:
+                        running_futures.remove(finished_future)
 
-            if len(future_to_task) == self.number_of_threads:
-                finished, pending = concurrent.futures.wait(
-                    fs=future_to_task,
-                    timeout=None,
-                    return_when=concurrent.futures.FIRST_COMPLETED,
-                )
-                for finished_future in finished:
-                    del future_to_task[finished_future]
+                if self.interrupt_exception:
+                    break
 
-                    try:
-                        finished_future.result()
-                    except worker.WorkerInterrupt as exception:
-                        interrupt_exception = exception
+        if self.should_use_a_killer:
+            self.thread_killer.stop()
 
-                        break
-
-        for future in concurrent.futures.as_completed(future_to_task):
-            try:
-                finished_future.result()
-            except worker.WorkerInterrupt as exception:
-                if not interrupt_exception:
-                    interrupt_exception = exception
-
-        for thread_killer in self.thread_killers.values():
-            thread_killer.kill()
-        self.thread_killers.clear()
-
-        executor.shutdown(
-            wait=True,
-        )
-
-        if interrupt_exception:
-            raise interrupt_exception
+        if self.interrupt_exception:
+            raise self.interrupt_exception
 
     def execute_task(
         self,
@@ -81,7 +69,7 @@ class ThreadedExecutor(
         )
 
         try:
-            returned_value = self.worker.work(
+            returned_value = self.worker_object.work(
                 task=task,
             )
         except worker.WorkerTimedout as exception:
@@ -91,7 +79,7 @@ class ThreadedExecutor(
                 exception=exception,
             )
 
-            self.worker.handle_timeout(
+            self.worker_object.handle_timeout(
                 task=task,
             )
         except worker.WorkerRetry as exception:
@@ -101,7 +89,7 @@ class ThreadedExecutor(
                 exception=exception,
             )
 
-            self.worker.handle_retry(
+            self.worker_object.handle_retry(
                 task=task,
             )
         except worker.WorkerMaxRetries as exception:
@@ -111,7 +99,7 @@ class ThreadedExecutor(
                 exception=exception,
             )
 
-            self.worker.handle_max_retries(
+            self.worker_object.handle_max_retries(
                 task=task,
             )
         except worker.WorkerRequeue as exception:
@@ -121,7 +109,7 @@ class ThreadedExecutor(
                 exception=exception,
             )
 
-            self.worker.handle_requeue(
+            self.worker_object.handle_requeue(
                 task=task,
             )
         except worker.WorkerInterrupt as exception:
@@ -131,7 +119,7 @@ class ThreadedExecutor(
                 exception=exception,
             )
 
-            raise exception
+            self.interrupt_exception = exception
         except Exception as exception:
             self.post_work(
                 task=task,
@@ -139,7 +127,7 @@ class ThreadedExecutor(
                 exception=exception,
             )
 
-            self.worker.handle_failure(
+            self.worker_object.handle_failure(
                 task=task,
                 exception=exception,
             )
@@ -149,7 +137,7 @@ class ThreadedExecutor(
                 success=True,
                 exception=None,
             )
-            self.worker.handle_success(
+            self.worker_object.handle_success(
                 task=task,
                 returned_value=returned_value,
             )
@@ -159,11 +147,11 @@ class ThreadedExecutor(
         task: objects.Task,
     ) -> None:
         try:
-            self.worker.pre_work(
+            self.worker_object.pre_work(
                 task=task,
             )
         except Exception as exception:
-            self.worker.logger.error(
+            self.worker_object.logger.error(
                 msg=f'pre_work has failed: {exception}',
                 extra={
                     'task': task,
@@ -171,18 +159,10 @@ class ThreadedExecutor(
             )
 
         if self.should_use_a_killer:
-            current_thread_id = threading.get_ident()
-
-            if current_thread_id in self.thread_killers:
-                self.thread_killers[current_thread_id].reset()
-                self.thread_killers[current_thread_id].resume()
-            else:
-                self.thread_killers[current_thread_id] = killer.thread.Killer(
-                    thread_id=current_thread_id,
-                    timeout=self.worker.config.timeouts.soft_timeout,
-                    exception=worker.WorkerSoftTimedout,
-                )
-                self.thread_killers[current_thread_id].start()
+            self.thread_killer.add(
+                thread_id=threading.get_ident(),
+                timeout=self.worker_object.config.timeouts.soft_timeout,
+            )
 
     def post_work(
         self,
@@ -191,16 +171,18 @@ class ThreadedExecutor(
         exception: typing.Optional[Exception] = None,
     ) -> None:
         if self.should_use_a_killer:
-            self.thread_killers[threading.get_ident()].suspend()
+            self.thread_killer.remove(
+                thread_id=threading.get_ident(),
+            )
 
         try:
-            self.worker.post_work(
+            self.worker_object.post_work(
                 task=task,
                 success=success,
                 exception=exception,
             )
         except Exception as exception:
-            self.worker.logger.error(
+            self.worker_object.logger.error(
                 msg=f'post_work has failed: {exception}',
                 extra={
                     'task': task,
@@ -208,11 +190,3 @@ class ThreadedExecutor(
                     'exception': exception,
                 },
             )
-
-    def __del__(
-        self,
-    ) -> None:
-        for thread_killer in self.thread_killers.values():
-            thread_killer.kill()
-
-        self.thread_killers.clear()
