@@ -34,10 +34,14 @@ class Worker:
         self.logger = logging.getLogger(
             name=self.config.name,
         )
-        self.logger.propagate = False
         self.logger.setLevel(
             level=self.config.logging.level,
         )
+        for handler in self.logger.handlers:
+            self.logger.removeHandler(
+                hdlr=handler,
+            )
+        self.logger.propagate = False
 
         if self.config.logging.log_to_stdout:
             stream_handler = logging.StreamHandler(
@@ -52,6 +56,10 @@ class Worker:
             self.logger.addHandler(
                 hdlr=stream_handler,
             )
+        elif not self.config.logging.handlers:
+            self.logger.addHandler(
+                hdlr=logging.NullHandler(),
+            )
 
         for handler in self.config.logging.handlers:
             self.logger.addHandler(
@@ -83,14 +91,14 @@ class Worker:
         self,
     ) -> None:
         self.executor_obj: executor._executor.Executor
-        if self.config.executor.type == 'serial':
+        if self.config.number_of_threads == 1:
             self.executor_obj = executor.serial.SerialExecutor(
-                worker=self,
+                worker_object=self,
             )
-        elif self.config.executor.type == 'threaded':
+        else:
             self.executor_obj = executor.threaded.ThreadedExecutor(
-                worker=self,
-                number_of_threads=self.config.executor.number_of_threads,
+                worker_object=self,
+                number_of_threads=self.config.number_of_threads,
             )
 
     def purge_tasks(
@@ -277,6 +285,16 @@ class Worker:
 
         try:
             self.initialize()
+        except WorkerRespawn:
+            summary['respawn'] = True
+            summary['end_time'] = datetime.datetime.utcnow()
+
+            return summary
+        except WorkerStop:
+            summary['stop'] = True
+            summary['end_time'] = datetime.datetime.utcnow()
+
+            return summary
         except Exception as exception:
             self.logger.error(
                 msg=f'initialize has failed: {exception}',
@@ -307,6 +325,10 @@ class Worker:
 
         try:
             self.finalize()
+        except WorkerRespawn:
+            summary['respawn'] = True
+        except WorkerStop:
+            summary['stop'] = True
         except Exception as exception:
             self.logger.error(
                 msg=f'finalize has failed: {exception}',
